#include "myworm.h"

MyWorm::MyWorm(int nSegments, int nSegmentSize)
:	m_nSegments(nSegments),
	m_nSegmentSize(nSegmentSize)
{
	// QImage texture("resources/images/alphatexture.png");
	// QBrush brush(texture);

	m_looks.setColor(QColor(0, 0, 255, 100));
	m_looks.setStyle(Qt::SolidLine);
	m_looks.setCapStyle(Qt::RoundCap);
	m_looks.setJoinStyle(Qt::RoundJoin);
	m_looks.setWidth(10);

	m_itemList = allocGraphicsItems(nSegments);
}

MyWorm::~MyWorm()
{
	m_itemList.clear();
}

const QPointF MyWorm::headPoint() const
{
	return m_segments.first();
}

const QPointF MyWorm::tailPoint() const
{
	return m_segments.last();
}

void MyWorm::setHeadPoint(const QPointF & head)
{
 	m_segments.clear();
	m_segments.prepend(head);
}

void MyWorm::follow(const QPointF & pos)
{
	QPointF head = headPoint();
	QPointF delta = head - pos;

	// this little calculation doesn't enforce a segment length of a specific
	// size, but ensures it's at least a certain length in one dimension
	if (qAbs(delta.x()) > m_nSegmentSize || qAbs(delta.y()) > m_nSegmentSize)
	{
		m_segments.prepend(pos);

		if (m_segments.size() > m_nSegments)
		{
			m_segments.takeLast();
		}
	
		updateGraphicsItems(m_segments, m_itemList);
	}
}

QList<QGraphicsLineItem *> MyWorm::allocGraphicsItems(int nSegments)
{
	QGraphicsLineItem *pItem = NULL;
	QList<QGraphicsLineItem *> itemList;
	
	for (int i = 0; i < nSegments; i++)
	{
		pItem = new QGraphicsLineItem(this);
		pItem->setPen(m_looks);

		addToGroup(pItem);
		itemList.append(pItem);
	}

	return itemList;
}

void MyWorm::updateGraphicsItems(	const QList<QPointF> & segments,
									QList<QGraphicsLineItem *> & pItems)
{
	int nNumOfSegments = segments.size();

	for (int i = 0; i < nNumOfSegments - 1; i++)
	{
		QPointF firstPoint = segments[i];
		QPointF secondPoint = segments[i+1];

		QGraphicsLineItem *pLineItem = pItems.value(i);
		pLineItem->setLine(QLineF(firstPoint, secondPoint));
	}
}

